#!/usr/bin/env python
# -*- coding: UTF-8 -*-
#
# PyStore: Flat-file datastore for timeseries data
# https://github.com/ranaroussi/pystore
#
# Copyright 2018-2020 Ran Aroussi
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import os as os
from .utils import Path

DEFAULT_PATH = os.environ.get("PYSTORE_PATH", Path.home() / "pystore")
DEFAULT_PARTITION_SIZE = 99e+6  # ~99MB
PARTITION_SIZE = 99e+6  # ~99MB

# dask distributed
_SCHEDULER = None
_CLIENT = None
